@@ -31,25 +31,22 @@
 #include <glom/application.h>
 #include <glom/utils_ui.h> //For Utils::image_scale_keeping_ratio().
 #include <glom/mode_data/datawidget/cellcreation.h>
+#include <gtkmm/stock.h>
+#include <glibmm/main.h>
 #include <libglom/db_utils.h>
 
 #include <iostream> //For debug output.
 
-#ifdef GLOM_ENABLE_MAEMO
-//TODO: Remove this when we don't need to call C hildon functions:
-#include <hildon/hildon.h>
-#endif
 
 namespace Glom
 {
 
 DbAddDel::DbAddDel()
-: m_column_is_sorted(false),
+: Gtk::Box(Gtk::ORIENTATION_VERTICAL),
+  m_column_is_sorted(false),
   m_column_sorted_direction(false),
   m_column_sorted(0),
-#ifndef GLOM_ENABLE_MAEMO
   m_pMenuPopup(0),
-#endif //GLOM_ENABLE_MAEMO
   m_bAllowUserActions(true),
   m_bPreventUserSignals(false),
   m_bIgnoreTreeViewSignals(false),
@@ -60,10 +57,10 @@ DbAddDel::DbAddDel()
   m_validation_retry(false),
   m_allow_view(true),
   m_allow_view_details(false),
-#ifndef GLOM_ENABLE_MAEMO
   m_treeviewcolumn_button(0),
-#endif
-  m_fixed_cell_height(0)
+  m_fixed_cell_height(0),
+  m_rows_count_min(0),
+  m_rows_count_max(0)
 {
   set_prevent_user_signals();
   set_ignore_treeview_signals(true);
@@ -81,39 +78,25 @@ DbAddDel::DbAddDel()
   m_TreeView.get_accessible()->set_name(_("Table Content"));
 #endif
 
-  #ifndef GLOM_ENABLE_MAEMO
   m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
   m_ScrolledWindow.set_shadow_type(Gtk::SHADOW_IN);
   m_TreeView.set_fixed_height_mode(); //This allows some optimizations.
   m_TreeView.set_rules_hint();
   m_ScrolledWindow.add(m_TreeView);
   pack_start(m_ScrolledWindow);
-  #else
-  //Do not let the treeview emit activated as soon as a row is pressed.
-  //TODO: Allow this default maemo behaviour?
-  g_object_set(m_TreeView.gobj(), "hildon-ui-mode", HILDON_UI_MODE_NORMAL, (void*)0);
-
-  //Allow get_selected() and get_active() to work:
-  m_TreeView.set_column_selection_mode(Hildon::TOUCH_SELECTOR_SELECTION_MODE_SINGLE);
-  pack_start(m_TreeView);
-
-  m_TreeView.signal_changed().connect(sigc::mem_fun(*this, &DbAddDel::on_maemo_touchselector_changed));
-  #endif //GLOM_ENABLE_MAEMO
 
   m_TreeView.show();
 
   //Make sure that the TreeView doesn't start out only big enough for zero items.
-  m_TreeView.set_size_request(-1, 150);
+  set_height_rows(6, 6);
 
   //Allow the user to change the column order:
   //m_TreeView.set_column_drag_function( sigc::mem_fun(*this, &DbAddDel::on_treeview_column_drop) );
 
-  #ifndef GLOM_ENABLE_MAEMO
   m_TreeView.add_events(Gdk::BUTTON_PRESS_MASK); //Allow us to catch button_press_event and button_release_event
   m_TreeView.signal_button_press_event().connect_notify( sigc::mem_fun(*this, &DbAddDel::on_treeview_button_press_event) );
   m_TreeView.signal_columns_changed().connect( sigc::mem_fun(*this, &DbAddDel::on_treeview_columns_changed) );
   //signal_button_press_event().connect(sigc::mem_fun(*this, &DbAddDel::on_button_press_event_Popup));
-  #endif //GLOM_ENABLE_MAEMO
   //add_blank();
 
   setup_menu();
@@ -128,11 +111,16 @@ DbAddDel::DbAddDel()
 
 #ifdef GLOM_ENABLE_CLIENT_ONLY //Actually this has only been necessary for Maemo.
   // Adjust sizing when style changed
-  // TODO_Maemo: This calls construct_specified_columns(), which runs the SQL query again.
-  //       Try to change the row and column sizes without doing that.
 
   signal_style_changed().connect(sigc::mem_fun(*this, &DbAddDel::on_self_style_changed));
 #endif // !GLOM_ENABLE_CLIENT_ONLY
+
+  Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_TreeView.get_selection();
+  if(refSelection)
+  {
+    refSelection->signal_changed().connect(
+      sigc::mem_fun(*this, &DbAddDel::on_treeview_selection_changed));
+  }
 }
 
 DbAddDel::~DbAddDel()
@@ -146,6 +134,23 @@ DbAddDel::~DbAddDel()
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 }
 
+void DbAddDel::set_height_rows(gulong rows_count_min, gulong rows_count_max)
+{
+  m_rows_count_min = rows_count_min;
+  m_rows_count_max = rows_count_max;
+
+  set_height_rows_actual(m_rows_count_min);
+}
+
+void DbAddDel::set_height_rows_actual(gulong rows_count)
+{
+  //TODO: File a bug about API for this in GtkTreeView.
+  const guint height_for_rows = rows_count * get_fixed_cell_height();
+  //std::cout << "debug: height_for_rows = " << height_for_rows << std::endl;
+  const guint extra_for_treeview = 50; //TODO: Find some way to guess this.
+  m_ScrolledWindow.set_min_content_height(height_for_rows + extra_for_treeview);
+}
+
 void DbAddDel::do_user_requested_edit()
 {
   Gtk::TreeModel::iterator iter = get_item_selected();
@@ -157,8 +162,6 @@ void DbAddDel::do_user_requested_edit()
     std::cerr << G_STRFUNC << ": No item was selected." << std::endl;
 }
 
-#ifndef GLOM_ENABLE_MAEMO
-
 void DbAddDel::on_idle_row_edit()
 {
   on_MenuPopup_activate_Edit();
@@ -210,29 +213,13 @@ void DbAddDel::on_MenuPopup_activate_Delete()
     }
   }
 }
-#endif //GLOM_ENABLE_MAEMO
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 void DbAddDel::on_MenuPopup_activate_layout()
 {
   finish_editing();
   signal_user_requested_layout().emit();
 }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
-#ifdef GLOM_ENABLE_MAEMO
-void DbAddDel::on_maemo_touchselector_changed(int /* column */)
-{
-  if(!m_bIgnoreTreeViewSignals)
-    do_user_requested_edit();
-}
-#endif //GLOM_ENABLE_MAEMO
-
-#ifdef GLOM_ENABLE_MAEMO
-void DbAddDel::setup_menu()
-{
-}
-#else
 void DbAddDel::setup_menu()
 {
   m_refActionGroup = Gtk::ActionGroup::create();
@@ -336,7 +323,7 @@ bool DbAddDel::on_button_press_event_Popup(GdkEventButton *event)
 #endif
 
   GdkModifierType mods;
-  gdk_window_get_pointer( gtk_widget_get_window (Gtk::Widget::gobj()), 0, 0, &mods );
+  gdk_window_get_device_position( gtk_widget_get_window(Gtk::Widget::gobj()), event->device, 0, 0, &mods );
   if(mods & GDK_BUTTON3_MASK)
   {
     //Give user choices of actions on this item:
@@ -358,7 +345,6 @@ bool DbAddDel::on_button_press_event_Popup(GdkEventButton *event)
 
   return  false; //Not handled. TODO: Call base class?
 }
-#endif //GLOM_ENABLE_MAEMO
 
 Gtk::TreeModel::iterator DbAddDel::get_item_placeholder()
 {
@@ -411,12 +397,6 @@ Gnome::Gda::Value DbAddDel::get_value_selected(const sharedptr<const LayoutItem_
 
 Gtk::TreeModel::iterator DbAddDel::get_item_selected()
 {
-  #ifdef GLOM_ENABLE_MAEMO
-  //TODO: See bug https://bugs.maemo.org/show_bug.cgi?id=4640
-  //about the get_selected()/get_active() confusion.
-  return m_TreeView.get_selected(0);
-  //This doesn't seem to work for anything but the first item: return m_TreeView.get_active();
-  #else
   Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
   if(refTreeSelection)
   {
@@ -427,19 +407,10 @@ Gtk::TreeModel::iterator DbAddDel::get_item_selected()
     return m_refListStore->children().end();
   else
     return Gtk::TreeModel::iterator();
-  #endif //GLOM_ENABLE_MAEMO
 }
 
 Gtk::TreeModel::iterator DbAddDel::get_item_selected() const
 {
-  #ifdef GLOM_ENABLE_MAEMO
-  Hildon::TouchSelector& unconst = const_cast<Hildon::TouchSelector&>(m_TreeView);
-  return unconst.get_selected(0);
-
-  //TODO: What would this mean?
-  //See https://bugs.maemo.org/show_bug.cgi?id=4641
-  // return m_TreeView.get_active();
-  #else
   Glib::RefPtr<const Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
   if(refTreeSelection)
   {
@@ -451,7 +422,6 @@ Gtk::TreeModel::iterator DbAddDel::get_item_selected() const
     return m_refListStore->children().end();
   else
     return Gtk::TreeModel::iterator();
-  #endif //GLOM_ENABLE_MAEMO
 }
 
 
@@ -508,9 +478,6 @@ bool DbAddDel::select_item(const Gtk::TreeModel::iterator& iter, const sharedptr
 
     treemodel_col += get_count_hidden_system_columns();
 
-    #ifdef GLOM_ENABLE_MAEMO
-    m_TreeView.select_iter(0, iter, true);
-    #else
     Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
     g_assert(refTreeSelection);
     refTreeSelection->select(iter);
@@ -524,9 +491,7 @@ bool DbAddDel::select_item(const Gtk::TreeModel::iterator& iter, const sharedptr
       Gtk::TreeView::Column* pColumn = m_TreeView.get_column(view_column_index);
       if(pColumn)
       {
-        #ifndef GLOM_ENABLE_MAEMO
         if(pColumn != m_treeviewcolumn_button) //This would activate the button. Let's avoid this, though it should never happen.
-        #endif //GLOM_ENABLE_MAEMO
         {
           m_TreeView.set_cursor(path, *pColumn, start_editing);
         }
@@ -536,7 +501,6 @@ bool DbAddDel::select_item(const Gtk::TreeModel::iterator& iter, const sharedptr
     }
     else
        g_warning("DbAddDel::select_item:TreeViewColumn index not found. column=%d", treemodel_col);
-    #endif //GLOM_ENABLE_MAEMO
 
     bResult = true;
   }
@@ -554,50 +518,18 @@ guint DbAddDel::get_count() const
   //Take account of the extra blank for new entries:
   if(get_allow_user_actions()) //If it has the extra row.
   {
-    if(get_is_placeholder_row(get_last_row()))
-      --iCount;
+    --iCount;
   }
 
   return iCount;
 }
 
-#ifdef GLOM_ENABLE_MAEMO
-Glib::RefPtr<Hildon::TouchSelectorColumn> DbAddDel::touch_selector_get_column()
-{
-  if(m_TreeView.get_num_columns() == 0)
-  {
-    //TODO: Needs a newer hildonmm: m_TreeView.append_column(m_refListStore);
-    hildon_touch_selector_append_column(m_TreeView.gobj(), m_refListStore->gobj(), 0, static_cast<char*>(0), true);
-  }
-
-  return m_TreeView.get_column(0);
-}
-
-Glib::RefPtr<const Hildon::TouchSelectorColumn> DbAddDel::touch_selector_get_column() const
-{
-  if(m_TreeView.get_num_columns() == 0)
-    return Glib::RefPtr<const Hildon::TouchSelectorColumn>();
-
-  return m_TreeView.get_column(0);
-}
-
-#endif //GLOM_ENABLE_MAEMO
-
 guint DbAddDel::get_columns_count() const
 {
-  #ifdef GLOM_ENABLE_MAEMO
-  Glib::RefPtr<const Hildon::TouchSelectorColumn> column = touch_selector_get_column();
-  if(!column)
-    return 0;
-
-  std::list<const Gtk::CellRenderer*> cells = column->get_cells();
-  return cells.size();
-  #else
   return m_TreeView.get_columns().size();
-  #endif //GLOM_ENABLE_MAEMO
 }
 
-int DbAddDel::get_fixed_cell_height()
+guint DbAddDel::get_fixed_cell_height()
 {
   if(m_fixed_cell_height <= 0)
   {
@@ -607,7 +539,7 @@ int DbAddDel::get_fixed_cell_height()
     // when all columns are known.
 
     //Get a default:
-    Glib::RefPtr<Pango::Layout> refLayout = create_pango_layout("example");
+    Glib::RefPtr<Pango::Layout> refLayout = m_TreeView.create_pango_layout("ExampleEg");
     int width = 0;
     int height = 0;
     refLayout->get_pixel_size(width, height);
@@ -629,19 +561,26 @@ int DbAddDel::get_fixed_cell_height()
         continue;
 
       // Translators: This is just some example text used to discover an appropriate height for user-entered text in the UI. This text itself is never shown to the user.
-      Glib::RefPtr<Pango::Layout> refLayout = create_pango_layout(_("Example"));
+      Glib::RefPtr<Pango::Layout> refLayout = m_TreeView.create_pango_layout(_("ExampleEg"));
       const Pango::FontDescription font(font_name);
       refLayout->set_font_description(font);
       int width = 0;
       int height = 0;
       refLayout->get_pixel_size(width, height);
 
-      if(height > m_fixed_cell_height)
-        m_fixed_cell_height = height;
+      if(height > (int)m_fixed_cell_height)
+        m_fixed_cell_height = (guint)height;
     }
   }
+  
+  //We add extra spacing, because otherwise the bottom of letters such as "g" get cut off.
+  //We get this style property, which might be causing it. murrayc 
+  //TODO: Find out if this is reallyt the right way to calculate the correct height:
+  int extra_height = 0;
+  gtk_widget_style_get(GTK_WIDGET(m_TreeView.gobj()), "vertical-separator", &extra_height, (void*)0);
+  //std::cout << "debug: extra_height=" << extra_height << std::endl;
 
-  return m_fixed_cell_height;
+  return m_fixed_cell_height + extra_height;
 }
 
 
@@ -659,7 +598,6 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
   Gtk::CellRendererText* pCellRendererText = dynamic_cast<Gtk::CellRendererText*>(pCellRenderer);
   if(pCellRendererText)
   {
-    #ifndef GLOM_ENABLE_MAEMO //List views are non-editable on Maemo.
     //Connect to edited signal:
     if(item_field) //Only fields can be edited:
     {
@@ -672,7 +610,6 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
       pCellRendererText->signal_edited().connect(
         sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_cell_edited), model_column_index, data_model_column_index) );
     }
-    #endif //GLOM_ENABLE_MAEMO
   }
   else
   {
@@ -683,11 +620,9 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
 
       if(item_field) //Only fields can be edited:
       {
-        #ifndef GLOM_ENABLE_MAEMO //There's no direct editing via the list view on Maemo.
         //Connect to its signal:
         pCellRendererToggle->signal_toggled().connect(
           sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_cell_edited_bool), model_column_index, data_model_column_index ) );
-        #endif //GLOM_ENABLE_MAEMO
       }
     }
     else
@@ -706,10 +641,8 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
     sharedptr<const LayoutItem_Button> item_button = sharedptr<const LayoutItem_Button>::cast_dynamic(layout_item);
     if(item_button)
     {
-      #ifndef GLOM_ENABLE_MAEMO
       pCellButton->signal_clicked().connect(
         sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_cell_layout_button_clicked), model_column_index) );
-     #endif //GLOM_ENABLE_MAEMO
     }
   }
 
@@ -731,13 +664,7 @@ void DbAddDel::construct_specified_columns()
     m_refListStore.reset();
     if(m_table_name.empty())
     {
-      #ifdef GLOM_ENABLE_MAEMO
-      //TODO: Needs a newer hildonmm: m_TreeView.append_column(m_refListStore);
-      //TODO: Remove all previous columns?
-      hildon_touch_selector_append_column(m_TreeView.gobj(), m_refListStore->gobj(), 0, static_cast<char*>(0), true);
-      #else
       m_TreeView.set_model(m_refListStore); // clear old model from treeview
-      #endif
     }
     else
       show_hint_model();
@@ -747,25 +674,16 @@ void DbAddDel::construct_specified_columns()
   m_refListStore = DbTreeModel::create(m_found_set, m_column_items, m_allow_view, m_find_mode, m_FieldsShown);
   //m_FieldsShown is needed by Base_DB_Table_Data::record_new().
 
-  #ifdef GLOM_ENABLE_MAEMO
-  //Remove all View columns:
-  while(m_TreeView.get_num_columns())
-  {
-    m_TreeView.remove_column(0);
-  }
-  #else
   m_TreeView.set_model(m_refListStore); // clear old model from treeview
 
   //Remove all View columns:
   m_TreeView.remove_all_columns();
-  #endif
 
 
   //Add new View Colums:
   int model_column_index = 0; //Not including the hidden internal columns.
   int view_column_index = 0;
 
-  #ifndef GLOM_ENABLE_MAEMO
   {
     GlomCellRenderer_ButtonImage* pCellButton = Gtk::manage(new GlomCellRenderer_ButtonImage());
 
@@ -795,7 +713,6 @@ void DbAddDel::construct_specified_columns()
 
     ++view_column_index;
   }
-  #endif //GLOM_ENABLE_MAEMO
 
   bool no_columns_used = true;
   int data_model_column_index = 0; //-1 means undefined index.
@@ -858,20 +775,29 @@ void DbAddDel::construct_specified_columns()
   }
   else
   {
-    #ifndef GLOM_ENABLE_MAEMO
     //We must set this each time, because show_hint_model() might unset it:
     m_TreeView.set_fixed_height_mode(); //This allows some optimizations.
-    #endif //GLOM_ENABLE_MAEMO
   }
 
-
-
-  #ifndef GLOM_ENABLE_MAEMO
   m_TreeView.columns_autosize();
-  #endif
 
   //Make sure there's a blank row after the database rows that have just been added.
   //add_blank();
+
+  //Adjust the number of rows to show.
+  //This can change the amount of height requested for the widget.
+  //Show as many rows as needed, but not more than the maximum:
+  gulong total = 0; //ignored
+  gulong db_rows_count_found = 0;
+  Glib::RefPtr<DbTreeModel> refModelDerived = Glib::RefPtr<DbTreeModel>::cast_dynamic(m_refListStore);
+  if(refModelDerived)
+    refModelDerived->get_record_counts(total, db_rows_count_found);
+  
+  //+1 for the empty row:
+  gulong rows_count = std::min(m_rows_count_max,  db_rows_count_found + 1);
+  //Do not use less than the minimum:
+  rows_count = std::max(rows_count, m_rows_count_min);
+  set_height_rows_actual(rows_count);
 }
 
 bool DbAddDel::refresh_from_database()
@@ -1171,16 +1097,8 @@ void DbAddDel::set_column_choices(guint col, const type_vec_strings& vecStrings)
   const bool test = get_view_column_index(col, view_column_index);
   if(test)
   {
-    #ifdef GLOM_ENABLE_MAEMO
-    Glib::RefPtr<Hildon::TouchSelectorColumn> column = touch_selector_get_column();
-    g_assert(column);
-    std::vector<Gtk::CellRenderer*> list_renderers = column->get_cells();
-    g_assert(!list_renderers.empty());
-    CellRendererDbList* pCellRenderer = dynamic_cast<CellRendererDbList*>(list_renderers[0]);
-    #else
     CellRendererDbList* pCellRenderer =
       dynamic_cast<CellRendererDbList*>( m_TreeView.get_column_cell_renderer(view_column_index) );
-    #endif //GLOM_ENABLE_MAEMO
     if(pCellRenderer)
     {
       //Add the choices:
@@ -1204,9 +1122,7 @@ void DbAddDel::set_allow_add(bool val)
 {
   m_allow_add = val;
 
-  #ifndef GLOM_ENABLE_MAEMO
   m_refContextAdd->set_sensitive(val);
-  #endif //GLOM_ENABLE_MAEMO
 }
 
 void DbAddDel::set_allow_delete(bool val)
@@ -1359,7 +1275,6 @@ DbAddDel::type_signal_sort_clause_changed DbAddDel::signal_sort_clause_changed()
   return m_signal_sort_clause_changed;
 }
 
-#ifndef GLOM_ENABLE_MAEMO
 void DbAddDel::on_cell_layout_button_clicked(const Gtk::TreeModel::Path& path, int model_column_index)
 {
   if(!m_refListStore)
@@ -1642,6 +1557,13 @@ bool DbAddDel::on_treeview_column_drop(Gtk::TreeView* /* treeview */, Gtk::TreeV
   return true;
 }
 
+/* We do not let the developer resize the columns directly in the treeview
+ * because we cannot easily avoid this signal handler from being called just during the 
+ * intial size allocation.
+ * Anyway, this would be rather implicit anyway - people might not know that they are changing it in the document.
+ * The size can still be specified in the layout dialog.
+ */
+/*
 void DbAddDel::on_treeview_column_resized(int model_column_index, DbTreeViewColumnGlom* view_column)
 {
   if(!view_column)
@@ -1670,6 +1592,7 @@ void DbAddDel::on_treeview_column_resized(int model_column_index, DbTreeViewColu
   if(layout_item)
     layout_item->set_display_width(width);
 }
+*/
 
 void DbAddDel::on_treeview_column_clicked(int model_column_index)
 {
@@ -1728,7 +1651,6 @@ void DbAddDel::on_treeview_columns_changed()
     //TODO: If this is ever wanted: m_signal_user_reordered_columns.emit();
   }
 }
-#endif //GLOM_ENABLE_MAEMO
 
 bool DbAddDel::get_column_to_expand(guint& column_to_expand) const
 {
@@ -1752,7 +1674,7 @@ bool DbAddDel::get_column_to_expand(guint& column_to_expand) const
       {
         //Check that no specific width has been specified:
         const guint column_width = layout_item_field->get_display_width();
-        if(!column_width) //TODO: Ignore these on maemo?
+        if(!column_width)
         {
           column_to_expand = i;
           result = true;
@@ -1771,7 +1693,6 @@ guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRend
 {
   InnerIgnore innerIgnore(this); //see comments for InnerIgnore class
 
-  #ifndef GLOM_ENABLE_MAEMO
   DbTreeViewColumnGlom* pViewColumn = Gtk::manage( new DbTreeViewColumnGlom(Utils::string_escape_underscores(title), cellrenderer) );
 
   //This is needed by fixed-height mode. We get critical warnings otherwise.
@@ -1779,16 +1700,6 @@ guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRend
   pViewColumn->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
 
   guint cols_count = m_TreeView.append_column(*pViewColumn);
-  #else
-  //Mathias Hasselmann says that this is required for the Maemo 5 style,
-  //though we don't know yet where that is documented. murrayc.
-  cellrenderer.set_property("xpad", HILDON_MARGIN_DEFAULT);
-
-  Glib::RefPtr<Hildon::TouchSelectorColumn> pViewColumn = touch_selector_get_column();
-  pViewColumn->pack_start(cellrenderer, expand);
-  g_assert(pViewColumn);
-  guint cols_count = get_columns_count();
-  #endif //GLOM_ENABLE_MAEMO
 
   sharedptr<const LayoutItem> layout_item = m_column_items[model_column_index];
   sharedptr<const LayoutItem_Field> layout_item_field = sharedptr<const LayoutItem_Field>::cast_dynamic(layout_item);
@@ -1800,20 +1711,16 @@ guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRend
       sigc::bind( sigc::mem_fun(*this, &DbAddDel::treeviewcolumn_on_cell_data), model_column_index, data_model_column_index) );
   }
 
-  #ifndef GLOM_ENABLE_MAEMO
   //Allow the column to be reordered by dragging and dropping the column header:
   pViewColumn->set_reorderable();
 
   //Allow the column to be resized:
   pViewColumn->set_resizable();
-  #endif //GLOM_ENABLE_MAEMO
 
-  #ifndef GLOM_ENABLE_MAEMO
   //GtkTreeView's fixed-height-mode does not allow us to have anything but
   //the last column as expandable.
   //TODO: Can we get the total size and calculate a starting size instead?
   expand = false;
-  #endif
 
   int column_width = -1; //Means expand.
   if(!expand)
@@ -1824,8 +1731,8 @@ guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRend
       //TODO: Choose a width based on the first 100 values.
       if(layout_item_field)
       {
-       column_width = Utils::get_suitable_field_width_for_widget(*this, layout_item_field, true /* or_title */);
-       column_width = column_width / 3;
+       column_width = Utils::get_suitable_field_width_for_widget(*this, layout_item_field, true /* or_title */, true /* for treeview */);
+       column_width = column_width + 8; //Some extra for the GtkTreeView's padding.
        //std::cout << "DEBUG: column_width=" << column_width << std::endl;
       }
       else
@@ -1833,9 +1740,6 @@ guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRend
     }
   }
 
-  #ifdef GLOM_ENABLE_MAEMO
-  cellrenderer.set_property("width", column_width);
-  #else
   if(column_width > 0) //Otherwise there's an assertion fails.
     pViewColumn->set_fixed_width(column_width); //This is the only way to set the width, so we need to set it as resizable again immediately afterwards.
 
@@ -1853,8 +1757,8 @@ guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRend
   pViewColumn->signal_clicked().connect(
     sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_column_clicked), model_column_index) );
 
-  pViewColumn->connect_property_changed("width", sigc::bind(sigc::mem_fun(*this, &DbAddDel::on_treeview_column_resized), model_column_index, pViewColumn) );
-  #endif //GLOM_ENABLE_MAEMO
+  // See the comment on on_treeview_column_resized():
+  //pViewColumn->connect_property_changed("width", sigc::bind(sigc::mem_fun(*this, &DbAddDel::on_treeview_column_resized), model_column_index, pViewColumn) );
 
   return cols_count;
 }
@@ -1935,11 +1839,6 @@ bool DbAddDel::get_is_placeholder_row(const Gtk::TreeModel::iterator& iter) cons
 {
   //g_warning("DbAddDel::get_is_placeholder_row()");
 
-  if(!get_is_last_row(iter))
-  {
-    return false;
-  }
-
   if(!iter)
     return false;
 
@@ -1974,14 +1873,12 @@ bool DbAddDel::get_view_column_index(guint model_column_index, guint& view_colum
 
   view_column_index = model_column_index;
 
-  #ifndef GLOM_ENABLE_MAEMO
   if(m_treeviewcolumn_button)
   {
     ++view_column_index;
   }
   else
     std::cout << "m_treeviewcolumn_button is null." << std::endl;
-  #endif //GLOM_ENABLE_MAEMO
 
   return true;
 }
@@ -2003,19 +1900,8 @@ void DbAddDel::set_key_field(const sharedptr<Field>& field)
   m_key_field = field;
 }
 
-#ifdef GLOM_ENABLE_MAEMO
-void DbAddDel::treeviewcolumn_on_cell_data(const Gtk::TreeModel::iterator& iter, int model_column_index, int data_model_column_index)
-#else
 void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter, int model_column_index, int data_model_column_index)
-#endif
 {
-#ifdef GLOM_ENABLE_MAEMO
-  Glib::RefPtr<Hildon::TouchSelectorColumn> column = touch_selector_get_column();
-  g_assert(column);
-  std::vector<Gtk::CellRenderer*> cells = column->get_cells();
-  Gtk::CellRenderer* renderer = cells[model_column_index];
-#endif
-
   //std::cout << "debug: DbAddDel::treeviewcolumn_on_cell_data()" << std::endl;
 
   if(iter)
@@ -2133,12 +2019,10 @@ void DbAddDel::set_allow_view_details(bool val)
 {
   m_allow_view_details = val;
 
-  #ifndef GLOM_ENABLE_MAEMO
   //Hide it if it was visible, if it exists,
   //otherwise do that later after creating it:
   if(m_treeviewcolumn_button)
     m_treeviewcolumn_button->set_visible(val);
-  #endif //GLOM_ENABLE_MAEMO
 }
 
 bool DbAddDel::get_allow_view_details() const
@@ -2165,28 +2049,16 @@ void DbAddDel::set_open_button_title(const Glib::ustring& title)
 
 void DbAddDel::show_hint_model()
 {
-  #ifdef GLOM_ENABLE_MAEMO
-  Glib::RefPtr<Hildon::TouchSelectorColumn> column = touch_selector_get_column();
-  g_assert(column);
-  column->clear();
-  #else
   m_TreeView.remove_all_columns();
   m_treeviewcolumn_button = 0; //When we removed the view columns, this was deleted because it's manage()ed.
-  #endif //GLOM_ENABLE_MAEMO
 
   m_model_hint = Gtk::ListStore::create(m_columns_hint);
   Gtk::TreeModel::iterator iter = m_model_hint->append();
   (*iter)[m_columns_hint.m_col_hint] = _("Right-click to layout, to specify the related fields.");
 
-  #ifdef GLOM_ENABLE_MAEMO
-  column = touch_selector_get_column();
-  g_assert(column);
-  column->pack_start(m_columns_hint.m_col_hint);
-  #else
   m_TreeView.set_model(m_model_hint);
   m_TreeView.set_fixed_height_mode(false); //fixed_height mode is incompatible with the default append_column() helper method.
   m_TreeView.append_column("", m_columns_hint.m_col_hint);
-  #endif
 }
 
 bool DbAddDel::start_new_record()
@@ -2385,7 +2257,7 @@ void DbAddDel::user_changed(const Gtk::TreeModel::iterator& row, guint col)
     else
     {
       //A field value was entered, but the record has not been added yet, because not enough information exists yet.
-       g_warning("Box_Data_List::on_adddel_user_changed(): debug: record not yet added.");
+      std::cout << G_STRFUNC << ": debug: record not yet added." << std::endl;
     }
   }
 }
@@ -2424,10 +2296,12 @@ void DbAddDel::user_added(const Gtk::TreeModel::iterator& row)
     //Use the user-entered primary key value:
 
     //This only works when the primary key is already stored: primary_key_value = get_value_key(row);
-    //sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
-    //layout_item->set_full_field_details(field);
+    //primary_key_value = get_value_key_selected();
 
-    primary_key_value = get_value_key_selected();
+    sharedptr<LayoutItem_Field> layout_field = sharedptr<LayoutItem_Field>::create();
+    layout_field->set_full_field_details(primary_key_field);
+    primary_key_value = get_value_selected(layout_field);
+    std::cout << "DEBUG: get_value_key_selected(): " << primary_key_value.to_string() << std::endl;
   }
 
   //If no primary key value is available yet, then don't add the record yet:
@@ -2542,4 +2416,20 @@ Gtk::TreeModel::iterator DbAddDel::get_row_selected()
   return get_item_selected();
 }
 
+void DbAddDel::on_treeview_selection_changed()
+{
+  Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_TreeView.get_selection();
+  if(!refSelection)
+    return;
+
+  const bool one_selected = (refSelection->count_selected_rows() > 0);
+  on_selection_changed(one_selected);
+}
+
+void DbAddDel::on_selection_changed(bool selection)
+{
+  m_refContextDelete->set_sensitive(selection);
+  m_refContextAdd->set_sensitive(selection);
+}
+
 } //namespace Glom
